@@ -16,34 +16,35 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed.cpp,v 1.109 2005/07/04 12:22:40 chpe Exp $
+ *  $Id: mozilla-embed.cpp,v 1.127 2006/05/10 18:07:00 chpe Exp $
  */
 
 #include "mozilla-config.h"
-
 #include "config.h"
 
-#include "mozilla-embed.h"
-#include "mozilla-embed-event.h"
-#include "ephy-embed-single.h"
-#include "ephy-embed-shell.h"
-#include "ephy-command-manager.h"
-#include "ephy-string.h"
-#include "ephy-debug.h"
-
-#include "EphyBrowser.h"
-#include "EventContext.h"
-#include "EphyUtils.h"
+#include <nsStringAPI.h>
 
 #include <gtkmozembed.h>
-#undef MOZILLA_INTERNAL_API
-#include <nsEmbedString.h>
-#define MOZILLA_INTERNAL_API 1
-#include <nsMemory.h>
-#include <nsIURI.h>
+#include <nsIDOMKeyEvent.h>
+#include <nsIDOMMouseEvent.h>
 #include <nsIRequest.h>
+#include <nsIURI.h>
+#include <nsIWebNavigation.h>
 #include <nsIWebProgressListener.h>
-#include <nsGfxCIID.h>
+#include <nsMemory.h>
+
+#include "EphyBrowser.h"
+#include "EphyUtils.h"
+#include "EventContext.h"
+
+#include "ephy-command-manager.h"
+#include "ephy-debug.h"
+#include "ephy-embed-shell.h"
+#include "ephy-embed-single.h"
+#include "ephy-string.h"
+#include "mozilla-embed-event.h"
+
+#include "mozilla-embed.h"
 
 static void	mozilla_embed_class_init	(MozillaEmbedClass *klass);
 static void	mozilla_embed_init		(MozillaEmbed *gs);
@@ -176,9 +177,28 @@ mozilla_embed_get_type (void)
 }
 
 static void
-impl_activate (EphyEmbed *embed) 
+mozilla_embed_grab_focus (GtkWidget *widget)
 {
-	gtk_widget_grab_focus (GTK_BIN (embed)->child);
+	GtkWidget *child;
+
+	child = gtk_bin_get_child (GTK_BIN (widget));
+
+	if (child != NULL)
+	{
+		gtk_widget_grab_focus (child);
+	}
+	else
+	{
+		g_warning ("Need to realize the embed before grabbing focus!\n");
+	}
+}
+
+static void
+impl_close (EphyEmbed *embed) 
+{
+	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED (embed)->priv;
+
+	mpriv->browser->Close ();
 }
 
 static void
@@ -186,14 +206,15 @@ mozilla_embed_realize (GtkWidget *widget)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED (widget)->priv;
 
-	(* GTK_WIDGET_CLASS(parent_class)->realize) (widget);
+	GTK_WIDGET_CLASS (parent_class)->realize (widget);
 
+	/* Initialise our helper class */
 	nsresult rv;
 	rv = mpriv->browser->Init (GTK_MOZ_EMBED (widget));
-
 	if (NS_FAILED (rv))
 	{
-               	g_warning ("EphyBrowser initialization failed for %p\n", widget);
+		g_warning ("EphyBrowser initialization failed for %p\n", widget);
+		return;
 	}
 }
 
@@ -224,6 +245,7 @@ mozilla_embed_class_init (MozillaEmbedClass *klass)
 
 	gtk_object_class->destroy = mozilla_embed_destroy;
 
+	widget_class->grab_focus = mozilla_embed_grab_focus;
 	widget_class->realize = mozilla_embed_realize;
 
 	g_type_class_add_private (object_class, sizeof(MozillaEmbedPrivate));
@@ -235,25 +257,25 @@ mozilla_embed_init (MozillaEmbed *embed)
         embed->priv = MOZILLA_EMBED_GET_PRIVATE (embed);
 	embed->priv->browser = new EphyBrowser ();
 
-	g_signal_connect_object (G_OBJECT (embed), "location",
+	g_signal_connect_object (embed, "location",
 				 G_CALLBACK (mozilla_embed_location_changed_cb),
 				 embed, (GConnectFlags) 0);
-	g_signal_connect_object (G_OBJECT (embed), "net_state_all",
+	g_signal_connect_object (embed, "net_state_all",
 				 G_CALLBACK (mozilla_embed_net_state_all_cb),
 				 embed, (GConnectFlags) 0);
-	g_signal_connect_object (G_OBJECT (embed), "dom_mouse_click",
+	g_signal_connect_object (embed, "dom_mouse_click",
 				 G_CALLBACK (mozilla_embed_dom_mouse_click_cb),
 				 embed, (GConnectFlags) 0);
-	g_signal_connect_object (G_OBJECT (embed), "dom_mouse_down",
+	g_signal_connect_object (embed, "dom_mouse_down",
 				 G_CALLBACK (mozilla_embed_dom_mouse_down_cb),
 				 embed, (GConnectFlags) 0);
-	g_signal_connect_object (G_OBJECT (embed), "dom-key-press",
+	g_signal_connect_object (embed, "dom-key-press",
 				 G_CALLBACK (mozilla_embed_dom_key_press_cb),
 				 embed, (GConnectFlags) 0);
-	g_signal_connect_object (G_OBJECT (embed), "new_window",
+	g_signal_connect_object (embed, "new_window",
 				 G_CALLBACK (mozilla_embed_new_window_cb),
 				 embed, (GConnectFlags) 0);
-	g_signal_connect_object (G_OBJECT (embed), "security_change",
+	g_signal_connect_object (embed, "security_change",
 				 G_CALLBACK (mozilla_embed_security_change_cb),
 				 embed, (GConnectFlags) 0);
 }
@@ -323,15 +345,15 @@ impl_can_go_forward (EphyEmbed *embed)
 static gboolean
 mozilla_embed_get_uri_parent (MozillaEmbed *membed,
 			      const char *aUri,
-			      nsEmbedCString &aParent)
+			      nsCString &aParent)
 {
         nsresult rv;
-	nsEmbedCString encoding;
+	nsCString encoding;
 	rv = membed->priv->browser->GetEncoding (encoding);
 	if (NS_FAILED (rv)) return FALSE;
 
         nsCOMPtr<nsIURI> uri;
-        rv = EphyUtils::NewURI (getter_AddRefs(uri), nsEmbedCString(aUri), encoding.get());
+        rv = EphyUtils::NewURI (getter_AddRefs(uri), nsCString(aUri), encoding.get());
         if (NS_FAILED(rv) || !uri) return FALSE;
 
 	/* Don't support going 'up' with chrome url's, mozilla handily
@@ -339,12 +361,12 @@ mozilla_embed_get_uri_parent (MozillaEmbed *membed,
 	 * rdf/chrome/src/nsChromeProtocolHandler.cpp::NewURI()
 	 * (the Canonify() call)
 	 */
-	nsEmbedCString scheme;
+	nsCString scheme;
 	rv = uri->GetScheme (scheme);
 	if (NS_FAILED(rv) || !scheme.Length()) return FALSE;
 	if (strcmp (scheme.get(), "chrome") == 0) return FALSE;
 
-	nsEmbedCString path;
+	nsCString path;
 	rv = uri->GetPath(path);
 	if (NS_FAILED(rv) || !path.Length()) return FALSE;
 	if (strcmp (path.get (), "/") == 0) return FALSE;
@@ -355,12 +377,12 @@ mozilla_embed_get_uri_parent (MozillaEmbed *membed,
 	if (slash[1] == '\0')
 	{
 		/* ends with a slash - a directory, go to parent */
-		rv = uri->Resolve (nsEmbedCString(".."), aParent);
+		rv = uri->Resolve (nsCString(".."), aParent);
 	}
 	else
 	{
 		/* it's a file, go to the directory */
-		rv = uri->Resolve (nsEmbedCString("."), aParent);
+		rv = uri->Resolve (nsCString("."), aParent);
 	}
 
 	return NS_SUCCEEDED (rv);
@@ -376,7 +398,7 @@ impl_can_go_up (EphyEmbed *embed)
 	address = ephy_embed_get_location (embed, TRUE);
 	if (address == NULL) return FALSE;
 
-	nsEmbedCString parent;
+	nsCString parent;
 	result = mozilla_embed_get_uri_parent (membed, address, parent);
 	g_free (address);
 
@@ -394,7 +416,7 @@ impl_get_go_up_list (EphyEmbed *embed)
 	if (address == NULL) return NULL;
 
 	s = address;
-	nsEmbedCString parent;
+	nsCString parent;
 	while (mozilla_embed_get_uri_parent (membed, s, parent))
 	{
 		s = g_strdup (parent.get());
@@ -428,7 +450,7 @@ impl_go_up (EphyEmbed *embed)
 	if (uri == NULL) return;
 
 	gboolean rv;
-	nsEmbedCString parent_uri;
+	nsCString parent_uri;
 	rv = mozilla_embed_get_uri_parent (membed, uri, parent_uri);
 	g_free (uri);
 
@@ -476,11 +498,15 @@ impl_get_location (EphyEmbed *embed,
 
 	nsCOMPtr<nsIURI> furi;
 	rv = uri->Clone (getter_AddRefs (furi));
-	if (NS_FAILED (rv)) return NULL;
+	/* Some nsIURI impls return NS_OK even though they didn't put anything in the outparam!! */
+	if (NS_FAILED (rv) || !furi) furi.swap(uri);
 
-	furi->SetPassword (nsEmbedCString());
+	/* Hide password part */
+	nsCString user;
+	furi->GetUsername (user);
+	furi->SetUserPass (user);
 
-	nsEmbedCString url;
+	nsCString url;
 	furi->GetSpec (url);
 
 	return url.Length() ? g_strdup (url.get()) : NULL;
@@ -490,10 +516,7 @@ static void
 impl_reload (EphyEmbed *embed, 
              gboolean force)
 {
-#ifdef HAVE_GECKO_1_8
-	guint32 mflags;
-
-	mflags = GTK_MOZ_EMBED_FLAG_RELOADNORMAL;
+	guint32 mflags = GTK_MOZ_EMBED_FLAG_RELOADNORMAL;
 
 	if (force)
 	{
@@ -501,15 +524,6 @@ impl_reload (EphyEmbed *embed,
 	}
 
 	gtk_moz_embed_reload (GTK_MOZ_EMBED(embed), mflags);
-#else
-	/* Workaround for broken reload with frames, see mozilla bug
-	 * http://bugzilla.mozilla.org/show_bug.cgi?id=246392
-	 */
-	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED (embed)->priv;
-
-	mpriv->browser->Reload (force ? EphyBrowser::RELOAD_FORCE :
-					EphyBrowser::RELOAD_NORMAL);
-#endif
 }
 
 static void
@@ -549,6 +563,34 @@ impl_get_zoom (EphyEmbed *embed)
 	return 1.0;
 }
 
+static void
+impl_scroll_lines (EphyEmbed *embed,
+		   int num_lines)
+{
+	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
+
+	mpriv->browser->ScrollLines (num_lines);
+}
+
+static void
+impl_scroll_pages (EphyEmbed *embed,
+		   int num_pages)
+{
+	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
+
+	mpriv->browser->ScrollPages (num_pages);
+}
+
+static void
+impl_scroll_pixels (EphyEmbed *embed,
+		    int dx,
+		    int dy)
+{
+	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
+
+	mpriv->browser->ScrollPixels (dx, dy);
+}
+
 static int
 impl_shistory_n_items (EphyEmbed *embed)
 {
@@ -569,7 +611,7 @@ impl_shistory_get_nth (EphyEmbed *embed,
                        char **aTitle)
 {
 	nsresult rv;
-        nsEmbedCString url;
+        nsCString url;
 	PRUnichar *title;
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
 
@@ -586,8 +628,8 @@ impl_shistory_get_nth (EphyEmbed *embed,
 
 	if (title)
 	{
-		nsEmbedCString cTitle;
-		NS_UTF16ToCString (nsEmbedString(title),
+		nsCString cTitle;
+		NS_UTF16ToCString (nsString(title),
 				   NS_CSTRING_ENCODING_UTF8, cTitle);
 		*aTitle = g_strdup (cTitle.get());
 		nsMemory::Free (title);
@@ -619,6 +661,20 @@ impl_shistory_go_nth (EphyEmbed *embed,
 	mpriv->browser->GoToHistoryIndex (nth);
 }
 
+static void
+impl_shistory_copy (EphyEmbed *source,
+		    EphyEmbed *dest,
+		    gboolean copy_back,
+		    gboolean copy_forward,
+		    gboolean copy_current)
+{
+	MozillaEmbedPrivate *spriv = MOZILLA_EMBED(source)->priv;
+	MozillaEmbedPrivate *dpriv = MOZILLA_EMBED(dest)->priv;
+
+	spriv->browser->CopySHistory(dpriv->browser, copy_back,
+				     copy_forward, copy_current);
+}
+
 static void
 impl_get_security_level (EphyEmbed *embed,
                          EphyEmbedSecurityLevel *level,
@@ -631,7 +687,7 @@ impl_get_security_level (EphyEmbed *embed,
 
 	nsresult rv;
 	PRUint32 state;
-	nsEmbedCString desc;
+	nsCString desc;
 	rv = mpriv->browser->GetSecurityInfo (&state, desc);
 	if (NS_FAILED (rv)) return;
 
@@ -691,7 +747,7 @@ impl_set_encoding (EphyEmbed *embed,
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
 	nsresult rv;
-	nsEmbedCString currEnc;
+	nsCString currEnc;
 
 	g_return_if_fail (encoding != NULL);
 
@@ -705,15 +761,8 @@ impl_set_encoding (EphyEmbed *embed,
 		if (NS_FAILED (rv)) return;
 	}
 
-#ifdef HAVE_GECKO_1_8
 	gtk_moz_embed_reload (GTK_MOZ_EMBED (embed),
 			      GTK_MOZ_EMBED_FLAG_RELOADCHARSETCHANGE);
-#else
-	/* Workaround for broken reload with frames, see mozilla bug
-	 * http://bugzilla.mozilla.org/show_bug.cgi?id=246392
-	 */
-	mpriv->browser->Reload (EphyBrowser::RELOAD_ENCODING_CHANGE);
-#endif
 }
 
 static char *
@@ -721,7 +770,7 @@ impl_get_encoding (EphyEmbed *embed)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
 	nsresult rv;
-	nsEmbedCString encoding;
+	nsCString encoding;
 
 	rv = mpriv->browser->GetEncoding (encoding);
 
@@ -738,7 +787,7 @@ impl_has_automatic_encoding (EphyEmbed *embed)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
 	nsresult rv;
-	nsEmbedCString encoding;
+	nsCString encoding;
 
 	rv = mpriv->browser->GetForcedEncoding (encoding);
 
@@ -785,6 +834,17 @@ update_load_state (MozillaEmbed *membed, gint state)
 				       priv->browser->GetDocumentType ());
 	}
 
+	if (state & GTK_MOZ_EMBED_FLAG_RESTORING &&
+	    priv->load_state == MOZILLA_EMBED_LOAD_STARTED)
+	{
+		priv->load_state = MOZILLA_EMBED_LOAD_LOADING;
+
+		char *address;
+		address = gtk_moz_embed_get_location (GTK_MOZ_EMBED (membed));
+		g_signal_emit_by_name (membed, "ge-content-change", address);
+		g_free (address);
+	}
+
 	if (state & GTK_MOZ_EMBED_FLAG_IS_NETWORK)
 	{
 		if (state & GTK_MOZ_EMBED_FLAG_START)
@@ -843,6 +903,7 @@ mozilla_embed_net_state_all_cb (GtkMozEmbed *embed, const char *aURI,
 		{ GTK_MOZ_EMBED_FLAG_IS_REQUEST, EPHY_EMBED_STATE_IS_REQUEST },
 		{ GTK_MOZ_EMBED_FLAG_IS_DOCUMENT, EPHY_EMBED_STATE_IS_DOCUMENT },
 		{ GTK_MOZ_EMBED_FLAG_IS_NETWORK, EPHY_EMBED_STATE_IS_NETWORK },
+		{ GTK_MOZ_EMBED_FLAG_RESTORING, EPHY_EMBED_STATE_RESTORING },
 		{ 0, EPHY_EMBED_STATE_UNKNOWN }
 	};
 
@@ -856,7 +917,7 @@ mozilla_embed_net_state_all_cb (GtkMozEmbed *embed, const char *aURI,
 
 	update_load_state (membed, state);
 	
-	g_signal_emit_by_name (membed, "ge_net_state", aURI, estate);
+	g_signal_emit_by_name (membed, "ge_net_state", aURI, /* FIXME: (gulong) */ estate);
 }
 
 static gboolean
@@ -1068,13 +1129,17 @@ ephy_embed_iface_init (EphyEmbedIface *iface)
 	iface->reload = impl_reload;
 	iface->set_zoom = impl_set_zoom;
 	iface->get_zoom = impl_get_zoom;
+	iface->scroll_lines = impl_scroll_lines;
+	iface->scroll_pages = impl_scroll_pages;
+	iface->scroll_pixels = impl_scroll_pixels;
 	iface->shistory_n_items = impl_shistory_n_items;
 	iface->shistory_get_nth = impl_shistory_get_nth;
 	iface->shistory_get_pos = impl_shistory_get_pos;
 	iface->shistory_go_nth = impl_shistory_go_nth;
+	iface->shistory_copy = impl_shistory_copy;
 	iface->get_security_level = impl_get_security_level;
 	iface->show_page_certificate = impl_show_page_certificate;
-	iface->activate = impl_activate;
+	iface->close = impl_close;
 	iface->set_encoding = impl_set_encoding;
 	iface->get_encoding = impl_get_encoding;
 	iface->has_automatic_encoding = impl_has_automatic_encoding;
