@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-dialog.h,v 1.12 2004/04/18 14:18:20 chpe Exp $
+ *  $Id: ephy-dialog.h,v 1.16 2006/01/16 23:51:40 chpe Exp $
  */
 
 #ifndef EPHY_DIALOG_H
@@ -35,14 +35,15 @@ G_BEGIN_DECLS
 #define EPHY_IS_DIALOG_CLASS(k)		(G_TYPE_CHECK_CLASS_TYPE ((k), EPHY_TYPE_DIALOG))
 #define EPHY_DIALOG_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), EPHY_TYPE_DIALOG, EphyDialogClass))
 
-typedef struct EphyDialogClass		EphyDialogClass;
-typedef struct EphyDialog		EphyDialog;
-typedef struct EphyDialogPrivate	EphyDialogPrivate;
+typedef struct _EphyDialogClass		EphyDialogClass;
+typedef struct _EphyDialog		EphyDialog;
+typedef struct _EphyDialogPrivate	EphyDialogPrivate;
 
 typedef enum
 {
-	PT_NORMAL,
-	PT_AUTOAPPLY
+	PT_NORMAL	= 0,
+	PT_AUTOAPPLY	= 1 << 0,
+	PT_INVERTED	= 1 << 1
 } EphyDialogApplyType;
 
 typedef struct
@@ -53,7 +54,7 @@ typedef struct
 	GType data_type;
 } EphyDialogProperty;
 
-struct EphyDialogClass
+struct _EphyDialogClass
 {
 	GObjectClass parent_class;
 
@@ -71,7 +72,7 @@ struct EphyDialogClass
 	void	(* show)	(EphyDialog *dialog);
 };
 
-struct EphyDialog
+struct _EphyDialog
 {
 	GObject parent;
 
@@ -94,7 +95,7 @@ void		ephy_dialog_construct		(EphyDialog *dialog,
 void		ephy_dialog_add_enum		(EphyDialog *dialog,
 						 const char *id,
 						 guint n_items,
-						 const char **items);
+						 const char * const *items);
 
 void		ephy_dialog_set_data_column	(EphyDialog *dialog,
 						 const char *id,
@@ -121,6 +122,10 @@ void		ephy_dialog_set_modal		(EphyDialog *dialog,
 GtkWidget      *ephy_dialog_get_control		(EphyDialog *dialog,
 						 const char *property_id);
 
+void		ephy_dialog_get_controls	(EphyDialog *dialog,
+						 const char *first_property_id,
+						 ...);
+
 gboolean	ephy_dialog_get_value		(EphyDialog *dialog,
 						 const char *property_id,
 						 GValue *value);
